/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://gre/modules/utils.js");

// define the bookmark tree
var Book = {
	
	historyService : null,
	bookmarkService : null,
	livemarkService : null,
	faviconService : null,
	annotationService : null,

	RESULT_TYPE_URI			: 0,
	RESULT_TYPE_FOLDER		: 6,
	RESULT_TYPE_SEPARATOR	: 7,

	TOOLBAR		: null,
	MENUFOLDER	: null,
	ROOT : null,

	TYPE_BOOKMARK		: 0,
	TYPE_FOLDER			: 1,
	TYPE_LIVEMARK		: 2,
	TYPE_SEPARATOR		: 3,
	TYPE_UNKNOWN 		: 4,
	
	init : function(){
		this.historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
							.getService(Components.interfaces.nsINavHistoryService);

		this.bookmarkService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
							.getService(Components.interfaces.nsINavBookmarksService);

		this.livemarkService = Components.classes["@mozilla.org/browser/livemark-service;2"]
							.getService(Components.interfaces.nsILivemarkService);
		this.faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
							.getService(Components.interfaces.nsIFaviconService);
		this.annotationService = Components.classes["@mozilla.org/browser/annotation-service;1"]
							.getService(Components.interfaces.nsIAnnotationService);

		this.TOOLBAR = this.bookmarkService.toolbarFolder;
		this.MENUFOLDER	= this.bookmarkService.bookmarksMenuFolder;
		this.ROOT = this.bookmarkService.placesRoot;
	},
	
	_init : function(){
		this.historyService = null;
		this.bookmarkService = null;
		this.livemarkService = null;
		this.faviconService = null;
		this.annotationService = null;
		this.TOOLBAR = null;
		this.MENUFOLDER = null;
		this.ROOT = null;
	},

	//+------------------------------------------------------------------------------------
	// Base types
	//+------------------------------------------------------------------------------------

	getBookmarkOptions : function(){
		var options = this.historyService.getNewQueryOptions();
		options.queryType = options.QUERY_TYPE_BOOKMARKS; 
		options.expandQueries = true;
		options.includeHidden = true;
		return options;
	},

	getBookmarkQuery : function(folders){
		var query = this.historyService.getNewQuery();
		query.setFolders(folders, folders.length);
		
		return query;
	},

	getBookmark : function(aItemId){
		var query = this.getBookmarkQuery([this.ROOT]);
		query.uri = this.bookmarkService.getBookmarkURI(aItemId);
		var options = this.getBookmarkOptions();
		
		var result = this.historyService.executeQuery(query, options);
		var node = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		
		return this.toBaseType([], result);
	},
	
	getBookmarkByURL : function(url){
		var query = this.getBookmarkQuery([this.ROOT]);
		query.uri = url.nsiURI();
		var options = this.getBookmarkOptions();
		
		var result = this.historyService.executeQuery(query, options);
		var node = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		
		return this.toBaseType([], result);
	},
	
	getParent : function(url){
		var folders = [this.ROOT];
		
		var query = this.getBookmarkQuery(folders);
		var options = this.getBookmarkOptions();
		options.expandQueries = true;
		
		var result = this.historyService.executeQuery(query, options);
		var node = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		
		return this.findParent(node, url);
	},
	
	findParent : function(node, uri){
		var parent = null;
		if(PlacesUtils.nodeIsFolder(node)){
			node = node.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
			
			node.containerOpen = true;
			for(var i = 0, l = node.childCount ;i < l; i++){
				var found = this.findParent(node.getChild(i), uri);
				if(found != null) parent = found;
			}
			node.containerOpen = false;
		}else {
			if(node.uri == uri)
				parent = node.parent;
		}
		return parent;
	},
	
	update : function(array, tree){
		var folders = [this.ROOT];
		var queries = [];
		
		array.each(function(elem){
			var query = Book.getBookmarkQuery(folders);
			query.uri = elem.nsiURI();
			queries.push(query);
		});

		var options = this.getBookmarkOptions();
		var placeURIs = PlacesUtils.history.queriesToQueryString(queries, queries.length, options);
		
		tree.place = placeURIs;
	},

	// returns all bookmarks incklusive duplicated urls
	getResourceArray : function(){
		return this.fill([]);
	},
	
	// returns bookmarks without duplicated urls
	getResourceHash : function(){
		return this.fill(new ExtendedHash());
	},
	
	getResourceIDHash : function(){
		return this.fill(new ExtendedHash(), 'ID');
	},

	getResourceCallback : function(callback){
		if(!callback) throw 'Error: No callback function added!';
		return this.fill(callback);
	},

	fill : function(object, id){
		var options = this.getBookmarkOptions();
		var query = this.getBookmarkQuery([this.ROOT]);

		var result = this.historyService.executeQuery(query, options);
		return this.toBaseType(object, result, id);
	},
	
	toBaseType : function(object, result, id){
		var root = result.root.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
		
		root.containerOpen = true;
		for (var i = 0, l = root.childCount; i < l; i++) {		
			this.collectNodes(root.getChild(i), object, id);
		}
		root.containerOpen = false;
		
  		return object;
	},

	collectNodes : function(node, object, id){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				this.put(node, object, id);
				break;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(this.isLivemark(node)){
					this.put(node, object, id);
				}else {
					node.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);

					node.containerOpen = true;
					try{
						for (var i = 0, l = node.childCount; i < l; i++) {	
							this.collectNodes(node.getChild(i), object, id);
						}
					}catch(e){}
					node.containerOpen = false;
				}
				break;
			}
			default : {
				break;
			}
		}
	},

	put : function(node, object, id){
		if(Object.isArray(object)){
			object.push(node);
		}else {
			if(object instanceof ExtendedHash){
				if(id){
					object.put(node.itemId, node);
				}else {
					object.put(this.getURI(node), node);
				}
			}else {
				if(Object.isFunction(object)){
					object(node);
				}else {
					throw 'Unknown base type : (' + (typeof object) + ')';
				}
			}
		}
	},

	resolveURL : function(nsIUri){
		var host = nsIUri.host;
		var path = nsIUri.path;
		var protocol = "http://";
		return protocol + host + "/" + path;
	},

	//+------------------------------------------------------------------------------------
	// Add
	//+------------------------------------------------------------------------------------

	addBookmark : function(name, url){
		var folderID = this.bookmarkService.bookmarksMenuFolder;
		this.bookmarkService.insertBookmark(folderID, url.nsiURI(), -1, name);
	},

	//TODO
	addFolder : function(){

	},

	//TODO
	addLivemark : function(){

	},

	//TODO
	addSeparator : function(){

	},

	//+------------------------------------------------------------------------------------
	// Remove
	//+------------------------------------------------------------------------------------

	//TODO: test
	removeBookmark : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeFolder : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeLivemark : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeSeparator : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//+------------------------------------------------------------------------------------
	// Properties
	//+------------------------------------------------------------------------------------

	//TODO: test
	getType : function(node){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				return this.TYPE_BOOKMARK;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					return this.TYPE_LIVEMARK;
				}
				return this.TYPE_FOLDER;
			}
			case this.RESULT_TYPE_SEPARATOR : {
				return this.TYPE_SEPARATOR;
			}
			default : {
				return this.TYPE_UNKNOWN;
			}
		}
	},

	getName : function(node){
		return node ? (node.title ? node.title : "") : "";
	},

	getUrl : function(node){
		return node ? (node.uri ? node.uri : "") : "";
	},

	getFUrl : function(node){
		if(node){
			var url = this.livemarkService.getFeedURI(node.itemId);
			var furl = url ? this.resolveURL(url) : null;
			return furl ? furl : "";
		}
		return "";
	},

	getURI : function(node){
		if(node){
			if(this.isLivemark(node)){
				return this.getFUrl(node);
			}
			return this.getUrl(node);
		}
		return "";
	},

	getKeys : function(node){
		if(node){
			var keys = this.bookmarkService.getKeywordForBookmark(node.itemId);
			return keys ? keys : "";
		}
		return "";
	},

	getAddDate : function(node){
		return node ? (node.dateAdded ? node.dateAdded : -1) : -1;
	},

	getLMDate : function(node){
		return node ? (node.lastModified ? node.lastModified : -1) : -1;
	},

	//TODO
	getLVDate : function(node){

		return "";
	},

	getDesc : function(node){
		if(node){
			if (this.annotationService.itemHasAnnotation(node.itemId, "bookmarkProperties/description")){
				return this.annotationService.getItemAnnotation(node.itemId, "bookmarkProperties/description");
			}
		}
		return "";
	},

	getIcon : function(node){
		var uri = this.getURI(node).nsiURI();
		var iconUri = null;
		
		try {
			iconUri = this.faviconService.getFaviconForPage(uri);
		} catch(e) {}
		
		if(iconUri){
			var mimeType = {};
			var iconData = null;
			try {
				iconData = this.faviconService.getFaviconData(iconUri, mimeType, {});
			} catch(e) {}
				if (iconData && iconData.length > 0)
					return "data:" + mimeType.value + ";base64," + Base64.encode(iconData, true);
		}
		return "chrome://browser/skin/Bookmarks-folder.png";
	},
	
	getItemIDByURL : function(url){
		var uri = url.nsiURI();
		var itemIDs = this.bookmarkService.getBookmarkIdsForURI(uri, {});
		var realID = -1;
		
		if(itemIDs.length > 1){ // in case of duplicates
			for(var i = 0, l = itemIDs.length ;i < l; i++){
				var receivedURI = this.bookmarkService.getBookmarkURI(itemIDs[i]);
				if(receivedURI.spec == uri.spec){
					return itemIDs[i];
				}
			}
		}else if(itemIDs.length == 1){
			return itemIDs[0];
		}
		
		return null;
	},
	
	getNameByURL : function(url){
		var itemID = this.getItemIDByURL(url);
		if(itemID != null){
			return this.bookmarkService.getItemTitle(itemID);
		}
		
		return null;
	},
	
	getIconByURL : function(url){
		var uri = url.nsiURI();
		var iconUri = null;
		
		try {
			iconUri = this.faviconService.getFaviconForPage(uri);
		} catch(e) {}
		
		if(iconUri){
			var mimeType = {};
			var iconData = null;
			try {
				iconData = this.faviconService.getFaviconData(iconUri, mimeType, {});
			} catch(e) {}
				if (iconData && iconData.length > 0)
					return "data:" + mimeType.value + ";base64," + Base64.encode(iconData, true);
		}
		return "chrome://browser/skin/Bookmarks-folder.png";
	},
	
	isLivemarkItem : function(node){
		var parent = node.parent;
		parent.QueryInterface(Components.interfaces.nsINavHistoryResultNode);
		return PlacesUtils.nodeIsLivemarkContainer(parent);
	},
	
	isLivemark : function(node){
		node.QueryInterface(Components.interfaces.nsINavHistoryResultNode);
		return PlacesUtils.nodeIsLivemarkContainer(node);
	},
	
	isBookmarked : function(url){
		return PlacesUtils.bookmarks.isBookmarked(url.nsiURI());
	}

};

Book.init();
