/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const ID_DUPLICATETREE = 'DUPLICATETREE';
const ID_DUPLICATE_COUNT = 'DUPLICATE_COUNT';

BT.DuplicatePane = {

	loadDuplicatePrefs : function(){
		var tree = $(ID_DUPLICATETREE);
		tree.clear();
		
		var duplicates = Prefs.getBranch('dupBm').toHash();
		var bookmarks = Book.getResourceHash();

		duplicates.each(function(key, value){
			if(bookmarks.contains(value.uri)){
				var name = Book.getNameByURL(value.uri);
				var icon = Book.getIconByURL(value.uri);
				tree.add(name, value.uri, icon, key);	
			}
		});
		
		var itemCount = $(ID_DUPLICATE_COUNT);
		itemCount.value = '(' + tree.size + ')';
	},
	
	doDiscard : function(){ 
		var tree = $(ID_DUPLICATETREE);

		var itemSet = tree.selection;
		tree.unselect();
	
		var bookmarks = Book.getResourceHash();
		var duplicates = Prefs.getBranch('dupBm').toHash();
		
		while(!itemSet.isEmpty()){
			var item = itemSet.pop();
			var url = tree.getItemURL(item);
			var key = tree.getItemKey(item);
			
			//remove from tree/hashmap
			tree.remove(item); 
			if(bookmarks.contains(url))
				Book.removeBookmark(bookmarks.remove(url).itemId);
			
			if(duplicates.contains(key)){
				Prefs.remove(key);
				duplicates.remove(key);
			}
		}
		
		var itemCount = $(ID_DUPLICATE_COUNT);
		itemCount.value = '(' + tree.size + ')';
	}

};
