/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

String.empty = '';
String.LF = '\n';
String.SPACE = ' ';
String.TAB = '\t';

String.prototype.trim = function(){
	var start = -1, end = this.length;  
	while(this.charCodeAt(--end) < 33);  
	while(this.charCodeAt(++start) < 33);  
	return this.slice(start, end+1); 
};

String.prototype.setCharAt = function(i, c){
	if(i > this.length - 1 || i < 0) return this;
	return this.substr(0, i) + c + this.substr(i + 1);
};

String.prototype.equalsIgnoreCase = function(str){
	var str0 = this.toLowerCase();
	var str1 = str.toLowerCase();
	return str0 == str1;
};

String.prototype.contains = function(str){
	return this.indexOf(str) != -1;
};

String.prototype.nsiURI = function(){
	var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
	return ioService.newURI(this, null, null);
};

Number.prototype.isPositive = function(){
	return this >= 0;
};

Number.prototype.isNegative = function(){
	return this <= 0;
};

Number.prototype.isZero = function(){
	return this == 0;
};

Array.prototype.isEmpty = function(){
	return this.length == 0;
};

Array.prototype.isNotEmpty = function(){
	return this.length != 0;
};

var JSONUtils = {
	toJSON : function(object){
		return Object.toJSON(object);
	},
	
	fromJSON : function(json){
		return eval('(' + json + ')');
	}
};

function nsIMutableArray(){
	return Components.classes["@mozilla.org/array;1"]
		.createInstance(Components.interfaces.nsIMutableArray);
}

function nsIWindowMediator(type){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
	
	var w = [];
	var e = wm.getEnumerator(type);
	
	while(e.hasMoreElements())
		w.push(e.getNext());
	return w;
}

function nsIBrowserWindow(){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
	var mainWindow = wm.getMostRecentWindow("navigator:browser");
	//var browser = mainWindow.getBrowser();
	return mainWindow;
}

var Prefs = {

	root : 'bookmarkTools',

	instance : function(){
		return Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch);
	},
	
	service : function(){
		return Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
	},
	
	getFile : function(key){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService).
                      getBranch("bookmarkTools.");
		return prefs.getComplexValue(key, Components.interfaces.nsILocalFile);
	},
	
	getBool : function(key){
		var inst = Prefs.instance();
		return inst.getBoolPref(Prefs.root + '.' + key);
	},
	
	getChar : function(key){
		var inst = Prefs.instance();
		return inst.getCharPref(Prefs.root + '.' + key);
	},
	
	getInt : function(key){
		var inst = Prefs.instance();
		return inst.getIntPref(Prefs.root + '.' + key);
	},
	
	setFile : function(key, file){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService).
                      getBranch("bookmarkTools.");
		prefs.setComplexValue(key, Components.interfaces.nsILocalFile, file);
	},

	setBool : function(key, val){
		var inst = Prefs.instance();
		inst.setBoolPref(Prefs.root + '.' + key, val);
	},
	
	setChar : function(key, val){
		var inst = Prefs.instance();
		inst.setCharPref(Prefs.root + '.' + key, val);
	},
	
	setInt : function(key, val){
		var inst = Prefs.instance();
		inst.setIntPref(Prefs.root + '.' + key, val);
	},
	
	remove : function(key){
		var inst = Prefs.instance();
		inst.clearUserPref(Prefs.root + '.' + key);
	},
	
	removeAt : function(key, uri){
		var branch = Prefs.getBranch(key);
		for(var i = 0, l = branch.length ;i < l; i++){ //TODO: performance
			var number = branch[i];
			var str = Prefs.getChar(key + number);
			var json = JSONUtils.fromJSON(str);
			
			if(json.uri == uri){
				Prefs.remove(key + number);
				return true;
			}
		}
		return false;
	},
	
	clear : function(){
		var inst = Prefs.instance();
		while(!this.isEmpty())
			inst.clearUserPref(Prefs.root + '.' + this.key + this.pop());
	},
	
	clearAll : function(){
		Prefs.getBranch(String.empty).clear();
	},
	
	getBranch : function(key){
		var branch = Prefs.service().getBranch(Prefs.root + '.' + key).getChildList(String.empty, {});
		branch.toHash = Prefs.toHash;
		branch.clear = Prefs.clear;
		branch.key = key;
		return branch;
	},
	
	toHash : function(){
		var inst = Prefs.instance();
		var hash = new ExtendedHash(), k = this.key;
		this.each(function(elem){
			var _key = Prefs.root + '.' + k + elem;
			var _value = JSONUtils.fromJSON(inst.getCharPref(_key));
			hash.put(k + elem, _value);
		});
		return hash;
	}

};

function ExtendedHash(){
	var _hash = [];
	var _trash = [];
	var _length = 0;
	
	this.put = function(key, value){
		if(_hash[key]){
			_trash.push(value);
		}else {
			_hash[key] = value;
			_length++;
		}
	};
	
	this.get = function(key){
		return _hash[key];
	};
	
	this.remove = function(key){
		var elem;
		if(this.contains(key)){
			elem = _hash[key];
			delete _hash[key];
			_length--;
		}
		return elem;
	};
	
	this.contains = function(key){
		return typeof _hash[key] != 'undefined';
	};
	
	this.length = function(){
		return _length;
	};
	
	this.isEmpty = function(){
		return _length == 0;
	};
	
	this.shift = function(){
		for(var i in _hash){
			var val;
			if(_length != 0){
				val = _hash[i];
				delete _hash[i];
				_length--;
			}
			return val;
		}
	};
	
	this.each = function(f){
		if(_length != 0){
			for(var i in _hash){
				if(!(i in Array.prototype))
					f(i, _hash[i]);
			}
		}
	};
	
	this.trash = function(){
		return _trash;
	};
}

BT.IO = {

	generateFile : function(dir, filename){
		var file = dir.clone();
		file.append(filename);
		if(!file.exists())
			file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0666);
		return file;
	},
	
	getFileObject : function(){
		var file = null;
		if(Prefs.getBool("indexPathDefault")){
			file = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get("ProfD", Components.interfaces.nsIFile);
		}else {
			file = Prefs.getFile("indexPath");
		}
		return file;
	},
	
	getBuildDir : function(){
		var file = this.getFileObject();

		file.append("cl_index");
		file.append("build");
		
		if(!file.exists() || !file.isDirectory()) 
			file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

		return file.clone();
	},
	
	getSearchDir : function(){
		var file = this.getFileObject();

		file.append("cl_index");
		file.append("search");
		
		if(!file.exists() || !file.isDirectory()) 
			file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

		return file.clone();
	},
	
	move : function(bdir, sdir){
		var entries = bdir.directoryEntries;
		while(entries.hasMoreElements()){
			var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
			if(!file.isDirectory()){
				// move file without renaming
				file.moveTo(sdir, file.leafName);
			}
		}
	},
	
	clearDirectories : function(){
		this.clear(this.getSearchDir());
		this.clear(this.getBuildDir());
	},
	
	clearLogs : function(){
		var dir = this.getFileObject();
		dir.append("cl_index");
		dir.append("log");
		this.clear(dir);
	},
	
	clear : function(dir){
		if(dir.exists() && dir.isDirectory()){
			var entries = dir.directoryEntries;
			while(entries.hasMoreElements()){
				var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
				if(file.isDirectory()){
					file.remove(true);
				}else {
					file.remove(false);
				}
			}
		}
	},
	
	getDirectoryInfo : function(dir){
		var size = 0;
		var entries = dir.directoryEntries;
		
		while(entries.hasMoreElements()){
			var file = entries.getNext().QueryInterface(Components.interfaces.nsIFile);
			if(file.isDirectory()){
				size += BT.IO.getDirectoryInfo(file)
			}else {
				size += file.fileSize;
			}
		}
		return size;
	},
	
	formatBytes : function(filesize){
		if (filesize >= 1073741824) {
			filesize = this.number_format(filesize / 1073741824, 2, '.', '') + ' Gb';
		} else {
			if (filesize >= 1048576) {
				filesize = this.number_format(filesize / 1048576, 2, '.', '') + ' Mb';
			} else {
				if (filesize >= 1024) {
					filesize = this.number_format(filesize / 1024, 0) + ' Kb';
				} else {
					filesize = this.number_format(filesize, 0) + ' bytes';
				}
			}
		}
		return filesize;
	},
	
	number_format : function(number, decimals, dec_point, thousands_sep){
		// http://kevin.vanzonneveld.net
		// +   original by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)
		// +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		// +     bugfix by: Michael White (http://crestidg.com)
		// +     bugfix by: Benjamin Lupton
		// +     bugfix by: Allan Jensen (http://www.winternet.no)
		// +    revised by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)    
		// *     example 1: number_format(1234.5678, 2, '.', '');
		// *     returns 1: 1234.57     
	 
		var n = number, c = isNaN(decimals = Math.abs(decimals)) ? 2 : decimals;
		var d = dec_point == undefined ? "," : dec_point;
		var t = thousands_sep == undefined ? "." : thousands_sep, s = n < 0 ? "-" : "";
		var i = parseInt(n = Math.abs(+n || 0).toFixed(c)) + "", j = (j = i.length) > 3 ? j % 3 : 0;
		
		return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
	},
	
	writeLineToFile : function(file, value){
		if(file){
			var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
							.createInstance(Components.interfaces.nsIFileOutputStream);

			// use 0x02 | 0x10 to open file for appending.
			foStream.init(file, 0x02 | 0x08 | 0x10, 0666, 0); // write, create, truncate

			foStream.write(value, value.length);
			foStream.close();
		}
	},
	
	readFileLineByLine : function(file){
		// open an input stream from file
		var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].
							createInstance(Components.interfaces.nsIFileInputStream);
		istream.init(file, 0x01, 0444, 0);
		istream.QueryInterface(Components.interfaces.nsILineInputStream);

		// read lines into array
		var line = {}, lines = new ExtendedHash(), hasmore, count = 0;
		hasmore = istream.readLine(line);
		
		while(hasmore){
			lines.put(line.value, count); 
			hasmore = istream.readLine(line);
			count++;
		}

		istream.close();

		return lines;
	},
	
	readFile : function(file){
		var data = "";
		var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
								.createInstance(Components.interfaces.nsIFileInputStream);
		var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"]
								.createInstance(Components.interfaces.nsIScriptableInputStream);
		fstream.init(file, -1, 0, 0);
		sstream.init(fstream); 

		var str = sstream.read(4096);
		while (str.length > 0) {
		  data += str;
		  str = sstream.read(4096);
		}

		sstream.close();
		fstream.close();
		return data;
	}

};

var InstallRDF = {

	version : "",
	name : "",
	description : "",
	creator : "",
	homepageURL : "",
	appID : "maik.gollnick@gmx.de",

	init : function(){
		InstallRDF.parse();
	},
	
	parse : function(){
		var em = Components.classes["@mozilla.org/extensions/manager;1"]
				.getService(Components.interfaces.nsIExtensionManager);
		var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
						.getService(Components.interfaces.nsIRDFService);
		var rdfLiteral = Components.interfaces.nsIRDFLiteral;
		
		var file = em.getInstallLocation(InstallRDF.appID).getItemFile(InstallRDF.appID, "install.rdf");

		var p = "file:///" + file.path.replace(/\\/g, "/");
		var fileDS = rdfService.GetDataSourceBlocking(p);
				
		var e = fileDS.GetAllResources();
		while(e.hasMoreElements()) {
	    	var r =  e.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
			if(r.Value == "urn:mozilla:install-manifest"){

				var t0 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#version"), true);
				var t1 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#name"), true);
				var t2 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#description"), true);
				var t3 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#creator"), true);
				var t4 = fileDS.GetTarget(r, rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#homepageURL"), true);

				if(t0) InstallRDF.version = " v." + t0.QueryInterface(rdfLiteral).Value;
				if(t1) InstallRDF.name = t1.QueryInterface(rdfLiteral).Value;
				if(t2) InstallRDF.description = t2.QueryInterface(rdfLiteral).Value;
				if(t3) InstallRDF.creator = t3.QueryInterface(rdfLiteral).Value;
				if(t4) InstallRDF.homepageURL = t4.QueryInterface(rdfLiteral).Value;
			}
		}
	}
};

InstallRDF.init();
